//---------------------------------------------------------------------
//  Pass 0
//---------------------------------------------------------------------
float4x4 matWorldView;
float4x4 matProjection;
float3 LightDir;

struct VS_INPUT_0 
{
   float4 Position : POSITION;
   float3 Normal   : NORMAL;
   float2 TexCoord : TEXCOORD0;
      
};

struct VS_OUTPUT_0 
{
   float4 Position : POSITION0;
   float4 Diffuse  : COLOR;
   float2 TexCoord : TEXCOORD0;
   
};

VS_OUTPUT_0 vs_main_0( VS_INPUT_0 Input )
{
   VS_OUTPUT_0 Output;
   
   float3 L = normalize(LightDir);
   float3 P = mul(Input.Position, matWorldView);
   float3 N = normalize(mul(Input.Normal, (float3x3)matWorldView));
   
   Output.Position = mul( float4(P, 1), matProjection );
   Output.Diffuse = max(0, dot(N, L));
   Output.TexCoord = Input.TexCoord;
   
   return( Output );
   
}

float4 TextureAmbientColor;
float4 TextureDiffuseColor;

texture DiffuseMap;
sampler2D baseMap;

struct PS_OUTPUT_0 
{
   float4 Diffuse  : COLOR;
   float2 TexCoord : TEXCOORD0;
   
};

float4 ps_main_0(PS_OUTPUT_0 Input) : COLOR0
{   
   float4 color = tex2D(baseMap, Input.TexCoord) + (Input.Diffuse * TextureDiffuseColor) + TextureAmbientColor;
   //float4 color = tex2D(baseMap, Input.TexCoord) * Input.Diffuse + TextureAmbientColor + TextureDiffuseColor;
   return( color );
   
}

//---------------------------------------------------------------------
//  Pass 1
//---------------------------------------------------------------------
float4 GlowColor;
float4 GlowAmbient;
float GlowThickness;

//float4x4 matWorldView;
//float4x4 matProjection;

struct VS_INPUT_1 
{
   float4 Position : POSITION0;
   float3 Normal   : NORMAL0;
   
};

struct VS_OUTPUT_1 
{
   float4 Position : POSITION0;
   float4 Diffuse  : COLOR0;
   
};

VS_OUTPUT_1 vs_main_1( VS_INPUT_1 Input )
{
   VS_OUTPUT_1 Output;
   
   //Normal (view space)
   float3 N = normalize(mul(Input.Normal, (float3x3)matWorldView));
   //Displaced position(view space)
   float3 P = mul(Input.Position, matWorldView) + GlowThickness * N;
   // glow axis
   float3 A = float3(0, 0, 1);
   float Power;
   Power = dot(N, A);
   Power *= Power;
   Power -= 1;
   Power *= Power;
   // Power = (1 - (N dot A)^2)^2 = ((N dot A)^2 - 1)^2
   
   //Projected position
   Output.Position = mul( float4(P, 1), matProjection );
   // Diffuse color is a combination of glow color and glow ambient
   Output.Diffuse = GlowColor * Power + GlowAmbient;   
   
   return( Output );
   
}

struct PS_INPUT_1
{
   float4 Diffuse : COLOR0;
};

float4 ps_main_1(PS_INPUT_1 Input) : COLOR0
{   
   return( Input.Diffuse );
   
}

//=================================================================================================================================
//Technique
//=================================================================================================================================
technique TShader
{
    pass P0
    {
        VertexShader = compile vs_2_0 vs_main_0();
        PixelShader  = compile ps_2_0 ps_main_0();
        
		Texture[0] = (DiffuseMap);
		
		Minfilter[0] = LINEAR;
		Magfilter[0] = LINEAR;
		Mipfilter[0] = LINEAR;

		AlphaBlendEnable = FALSE;
    }
    pass P1
    {
        VertexShader = compile vs_2_0 vs_main_1();
        PixelShader  = compile ps_2_0 ps_main_1();
        
		Texture[0] = NULL;
		
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
    }
}