//=================================================================================================================================
//Pass 0
//=================================================================================================================================
float3 LightDir;
float4 AmbientColor;
float4 DiffuseColor;
float4x4 matProjection;
float4x4 matWorldView;

texture DiffuseMap;
sampler2D baseMap;

struct VS_INPUT_1
{
   float4 Position : POSITION0;
   float3 Normal :   NORMAL;
   float2 TexCoord : TEXCOORD0;
};
struct VS_OUTPUT_1
{
   float4 Position : POSITION0;
   float4 Diffuse :  COLOR;
   float2 TexCoord : TEXCOORD0;
};
struct PS_INPUT_1
{
   float4 Diffuse  : COLOR;
   float2 TexCoord : TEXCOORD0;
};

VS_OUTPUT_1 vsMain_1(VS_INPUT_1 Input)
{
   VS_OUTPUT_1 Output;

   float3 L = normalize(LightDir);
   float3 P = mul(Input.Position, matWorldView);
   float3 N = normalize( mul(Input.Normal, (float3x3)matWorldView) );
   
   Output.Position = mul( float4(P, 1), matProjection );
   Output.Diffuse = max(0, dot(N, L));
   Output.TexCoord = Input.TexCoord;

   return( Output );
}
float4 psMain_1(PS_INPUT_1 Input) : COLOR0
{
   float4 color = tex2D(baseMap, Input.TexCoord) * Input.Diffuse + AmbientColor + DiffuseColor;
   return( color );
}


//=================================================================================================================================
//Pass 1
//=================================================================================================================================
float4 GlowColor;
float4 GlowAmbient;
float GlowThickness;

struct VS_INPUT_2
{
   float4 Position : POSITION0;
   float3 Normal   : NORMAL;
};
struct VS_OUTPUT_2
{
   float4 Position : POSITION0;
   float4 Diffuse  : COLOR;
};

VS_OUTPUT_2 vsMain_2(VS_INPUT_2 Input)
{
   VS_OUTPUT_2 Output;

   float3 N = normalize( mul(Input.Normal, (float3x3)matWorldView) );
   float3 P = mul(Input.Position, matWorldView) + GlowThickness * N;

   float A = float3(0.0f, 0.0f, 1.0f);
   float Power;
   Power = dot(N, A);
   Power = Power * Power;
   Power = Power - 1.0f;
   Power = Power * Power;

   Output.Position = mul( float4(P, 1), matProjection );
   Output.Diffuse = GlowColor * Power + GlowAmbient;

   return( Output );
}
float4 psMain_2(float4 Diffuse : COLOR) : COLOR0
{
   return( Diffuse );
}


//=================================================================================================================================
//Technique
//=================================================================================================================================
technique TShader
{
    pass P0
    {
        VertexShader = compile vs_2_0 vsMain_1();
        PixelShader  = compile ps_2_0 psMain_1();
        
		Texture[0] = (DiffuseMap);
		
		Minfilter[0] = LINEAR;
		Magfilter[0] = LINEAR;
		Mipfilter[0] = LINEAR;

		AlphaBlendEnable = FALSE;
    }
    pass P1
    {
        VertexShader = compile vs_2_0 vsMain_2();
        PixelShader  = compile ps_2_0 psMain_2();
        
		Texture[0] = NULL;
		
		AlphaBlendEnable = TRUE;
		SrcBlend = ONE;
		DestBlend = ONE;
    }
}